/**
 * Modified red_pitaya_pid.v, Andrej Trost, december 2024 
 *
 * Demo component: downsampling, low pass FIR and amplitude scaling
 * Registers 
 *   Base address = 0x4030
 *   offset read    write   default
 *   0x50   set,ID  set     0 = disabled
 *   0x54   scale   scale   3-bit scaling factor   
 */

module red_pitaya_pid (
   // signals
   input                 clk_i           ,  //!< processing clock
   input                 rstn_i          ,  //!< processing reset - active low
   input      [ 14-1: 0] dat_a_i         ,  //!< input data CHA
   input      [ 14-1: 0] dat_b_i         ,  //!< input data CHB
   output     [ 14-1: 0] dat_a_o         ,  //!< output data CHA
   output     [ 14-1: 0] dat_b_o         ,  //!< output data CHB
  
   // system bus
   input      [ 32-1: 0] sys_addr        ,  //!< bus address
   input      [ 32-1: 0] sys_wdata       ,  //!< bus write data
   input                 sys_wen         ,  //!< bus write enable
   input                 sys_ren         ,  //!< bus read enable
   output reg [ 32-1: 0] sys_rdata       ,  //!< bus read data
   output reg            sys_err         ,  //!< bus error indicator
   output reg            sys_ack            //!< bus acknowledge signal
);

reg [10-1:0] cnt;
reg signed [24-1:0] sum;
reg signed [14-1:0] dat_a_average;

// Channel A downsampling
//  Calculate average value of 1024 points from channel A and
//  output clk_sample = clk_i/1024 = 122 kHz
always @(posedge clk_i) // 1024 point average
if (rstn_i == 1'b0) begin
   cnt <= 0 ;
end else begin
   cnt <= cnt + 1;   
   if (cnt==0) begin
      dat_a_average <= sum[23:10];
      sum <= $signed(dat_a_i);
   end else begin
      sum <= $signed(sum) + $signed(dat_a_i);
   end
end

wire clk_sample = cnt[9];
wire signed [14-1:0] dat_fir;

// Calculate FIR filter response
fir i_fir (
 .clk(clk_sample),
 .x(dat_a_average),
 .y(dat_fir)
);

// Data scaling with saturation
wire [17-1: 0] mul_1, mul_2;
reg  [14-1: 0] out_1, out_2;

reg [2:0] scale = 3'b001;   // scaling factor

assign mul_1 = $signed(dat_a_average) * $signed(scale);
assign mul_2 = $signed(dat_fir) * $signed(scale);

always @(posedge clk_i) begin   
  if ($signed(mul_1) > $signed(17'd8191))       // postitive saturation
     out_1 <= 14'h1FFF ;
  else if ($signed(mul_1) < $signed(-17'd8191)) // negative saturation
     out_1 <= 14'h2000 ;
  else
     out_1 <= mul_1[14-1:0] ;

  if ($signed(mul_2) > $signed(17'd8191))      // postitive saturation
     out_2 <= 14'h1FFF ;
  else if ($signed(mul_2) < $signed(-17'd8191)) // negative saturation
     out_2 <= 14'h2000 ;
  else
     out_2 <= mul_2[14-1:0] ;
end

reg set = 1'b0;

// Select output
assign dat_a_o = (set==1)? out_1 : dat_a_i;
assign dat_b_o = (set==1)? out_2 : dat_b_i;

//  System bus registers
always @(posedge clk_i) begin
   if (rstn_i == 1'b0) begin
      scale <= 3'b001;
   end
   else begin
      if (sys_wen) begin
         if (sys_addr[19:0]==16'h50)    set <= sys_wdata[0];
         if (sys_addr[19:0]==16'h54)    scale <= sys_wdata[3-1:0];         
      end
   end
end

wire sys_en;
assign sys_en = sys_wen | sys_ren;

always @(posedge clk_i)
if (rstn_i == 1'b0) begin
   sys_err <= 1'b0 ;
   sys_ack <= 1'b0 ;
end else begin
   sys_err <= 1'b0 ;

   casez (sys_addr[19:0]) 
      20'h50 : begin sys_ack <= sys_en;    sys_rdata <= {set, 31'h2024}; end
      20'h54 : begin sys_ack <= sys_en;    sys_rdata <= scale          ; end      
     default : begin sys_ack <= sys_en;    sys_rdata <=  32'h0         ; end
   endcase
end

endmodule